--------------------------------------------------------------------------------------------------------
--                                            Tanked Widget                                           --
--------------------------------------------------------------------------------------------------------
local _, ns = ...
local t = ns.TidyPlates_Threat

local path = "Interface\\AddOns\\TidyPlates_Threat\\Widgets\\TankedWidget\\"
local DB
local WidgetList = {}

--------------------------------------------------------------------------------------------------------
--                                         General functions                                          --
--------------------------------------------------------------------------------------------------------
local function UpdateSettings(frame)
	frame:SetFrameLevel(frame:GetParent():GetFrameLevel() + 2)
	frame:SetSize(DB.scale, DB.scale)
	frame:SetPoint("CENTER", frame:GetParent(), DB.anchor, DB.x, DB.y)
end

local function UpdateWidgetFrame(frame)
	local unit = frame.unit

	if (not InCombatLockdown()) or (not unit.isInCombat) or (unit.threatValue > 2) then
		WidgetList[unit.unitid] = nil
		frame:Hide()
		return
	else
		WidgetList[unit.unitid] = frame
	end

	if TidyPlatesWidgets.IsEnemyTanked(unit) then
		frame.Icon:SetTexture(path .. "tank")
	else
		frame.Icon:SetTexture(path .. "dps")
	end

	frame:Show()
end

--------------------------------------------------------------------------------------------------------
--                                           Events watcher                                           --
--------------------------------------------------------------------------------------------------------
local WatcherFrame = CreateFrame("frame")
local isEnabled = false

local function EnableWatcherFrame()
	if not isEnabled then
		isEnabled = true
		TidyPlatesWidgets.EnableTankWatch()
		WatcherFrame:RegisterEvent("UNIT_THREAT_SITUATION_UPDATE")
	end
end

local function DisableWatcherFrame()
	if isEnabled then
		isEnabled = false
		TidyPlatesWidgets.DisableTankWatch()
		WatcherFrame:UnregisterAllEvents()
		wipe(WidgetList)
	end
end

WatcherFrame:SetScript("OnEvent", function(this, event, ...)
	for unitid, frame in pairs(WidgetList) do
		frame.unit.threatValue = UnitThreatSituation("player", frame.unit.unitid) or 0
		frame.unit.isInCombat = UnitAffectingCombat(frame.unit.unitid)
		UpdateWidgetFrame(frame)
	end
end)

local function enabled()
	DB = TidyPlatesThreat.db.profile.tankedWidget
	if DB.ON then
		EnableWatcherFrame()
	else
		DisableWatcherFrame()
	end
	return DB.ON
end

--------------------------------------------------------------------------------------------------------
--                                          Widget functions                                          --
--------------------------------------------------------------------------------------------------------
local function UpdateTankedWidget(frame, unit)
	if (not enabled()) or (not unit.unitid) or t.IsFriend(unit.unitid) or unit.type == "PLAYER" then
		frame:Hide()
		return
	end

	UpdateSettings(frame)
	frame.unit = unit

	UpdateWidgetFrame(frame)
end

local function CreateTankedWidget(parent)
	local frame = CreateFrame("frame", nil, parent)
	frame:SetFrameStrata("LOW")
	frame.Icon = frame:CreateTexture(nil, "OVERLAY")
	frame.Icon:SetAllPoints(frame)
	frame:Show()
	frame:Hide()
	frame.Update = UpdateTankedWidget
	return frame
end

ThreatPlatesWidgets.RegisterWidget("TankedWidget", CreateTankedWidget, false, enabled)
